@@ -1,6 +1,6 @@
 from easytorch.config import default_ap, default_args
 from easytorch.data import ETDataset, ETDataHandle, UnPaddedDDPSampler
-from easytorch.metrics import ETMetrics, ETAverages, Prf1a, ConfusionMatrix
+from easytorch.metrics import ETMetrics, ETAverages, Prf1a, ConfusionMatrix, ETMeter
 
 from .easytorch import EasyTorch
 from .trainer import ETTrainer
